package com.jwl.driver.server.service.impl;

import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.dto.DriverSchoolDto;
import com.jwl.driver.server.dto.LoginUserDto;
import com.jwl.driver.server.dto.SecurityUser;
import com.jwl.driver.server.entity.TdSysUser;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.TdSysUserMapper;
import com.jwl.driver.server.redis.RedisCache;
import com.jwl.driver.server.service.ITdSysUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.util.DateTimeUtil;
import com.jwl.driver.server.util.SecurityUtil;
import com.jwl.driver.server.util.SmsUtil;
import com.jwl.driver.server.util.TokenThreadUtil;
import com.jwl.driver.server.vo.LoginUserVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 用户表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class TdSysUserServiceImpl extends ServiceImpl<TdSysUserMapper, TdSysUser> implements ITdSysUserService {

    @Autowired
    private RedisCache redisCache;

    @Value("${token.expireTime}")
    private Integer tokenExpireTime;

    @Value("${message.code.expireTime}")
    private Integer codeExpireTime;

    @Override
    public String code(String phone) {

        if (!PhoneUtil.isPhone(phone)) {
            throw new BusinessException("手机号格式不正确");
        }

        if (redisCache.hasKey(Constants.MESSAGE_CODE_PREFIX + phone)){
            Long expire = redisCache.getExpire(Constants.MESSAGE_CODE_PREFIX + phone);
            throw new BusinessException("请在" + expire + "秒后重新获取验证码");
        }

        String code = SmsUtil.sendMessage(phone);
        redisCache.setCacheObject((Constants.MESSAGE_CODE_PREFIX + phone), code, codeExpireTime, TimeUnit.SECONDS);
        return code;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public LoginUserVo login(LoginUserDto loginUserDto) {
        //校验验证码 如果是默认验证码则不需要校验
        if (!StrUtil.equals(Constants.DEFAULT_MESSAGE_CODE,loginUserDto.getCode())){
            String code = redisCache.getCacheObject(Constants.MESSAGE_CODE_PREFIX + loginUserDto.getPhone());

            if (StrUtil.isBlank(code)){
                throw new BusinessException("验证码已过期，请重新获取");
            }

            if (!StrUtil.equals(code,loginUserDto.getCode())){
                throw new BusinessException("验证码错误");
            }
        }

        LambdaQueryWrapper<TdSysUser> cond = new LambdaQueryWrapper<TdSysUser>()
                .eq(TdSysUser::getPhone, loginUserDto.getPhone());
        //用户不存在则直接注册登陆
        TdSysUser tdSysUser = this.baseMapper.selectOne(cond);
        //todo 用户基础信息填写这里还要完善一下
        if (tdSysUser == null) {
            tdSysUser = new TdSysUser()
                    .setUserName("车友")
                    .setAvatar("")
                    .setPhone(loginUserDto.getPhone())
                    .setCreateTime(LocalDateTime.now());
            int insert = this.getBaseMapper().insert(tdSysUser);
            if (insert != 1) {
                throw new BusinessException("用户注册异常");
            }
        }
        String token = UUID.randomUUID().toString();
        LoginUserVo loginUserVo = new LoginUserVo();
        BeanUtils.copyProperties(tdSysUser, loginUserVo);
        loginUserVo.setToken(token);
        loginUserVo.setDays(DateTimeUtil.getIntervalDays(loginUserVo.getCreateTime(),LocalDateTime.now()) + 1);
        // 存入redis
        redisCache.setCacheObject(token, tdSysUser, tokenExpireTime, TimeUnit.DAYS);
        return loginUserVo;
    }

    @Override
    public Boolean loginOut() {
        String token = TokenThreadUtil.getToken();
        if (StrUtil.isNotBlank(token)) {
            redisCache.deleteObject(token);
        }
        return Boolean.TRUE;
    }

    @Override
    public LoginUserVo queryUserMessage() {
        SecurityUser loginUser = SecurityUtil.getLoginUser();

        LambdaQueryWrapper<TdSysUser> cond = new LambdaQueryWrapper<TdSysUser>()
                .eq(TdSysUser::getUserId, loginUser.getUserId());

        TdSysUser tdSysUser = this.baseMapper.selectOne(cond);
        if (Objects.isNull(tdSysUser)){
            throw new BusinessException("用户不存在或者已被删除");
        }

        LoginUserVo loginUserVo = new LoginUserVo();
        BeanUtils.copyProperties(tdSysUser, loginUserVo);
        loginUserVo.setDays(DateTimeUtil.getIntervalDays(loginUserVo.getCreateTime(),LocalDateTime.now()) + 1);

        return loginUserVo;
    }

    @Override
    public Boolean bindSchool(DriverSchoolDto schoolDto) {
        SecurityUser loginUser = SecurityUtil.getLoginUser();

        LambdaQueryWrapper<TdSysUser> cond = new LambdaQueryWrapper<TdSysUser>()
                .eq(TdSysUser::getUserId, loginUser.getUserId());

        TdSysUser tdSysUser = this.baseMapper.selectOne(cond);
        if (Objects.isNull(tdSysUser)){
            throw new BusinessException("用户不存在或者已被删除");
        }
        tdSysUser.setSchoolId(schoolDto.getSchoolId())
                .setSchoolName(schoolDto.getSchoolName())
                .setSchoolPhone(schoolDto.getSchoolPhone());

        boolean result = this.updateById(tdSysUser);
        if (!result){
            throw new BusinessException("绑定驾校失败");
        }
        return Boolean.TRUE;
    }

}
